--- ../lrzip-0.42-orig/stream.c	2009-11-29 23:17:48.000000000 +0200
+++ stream.c	2009-12-25 16:19:58.000000000 +0200
@@ -19,9 +19,9 @@
 /* multiplex N streams into a file - the streams are passed
    through different compressors */
 
-#include "rzip.h"
 #include <bzlib.h>
 #include <zlib.h>
+#include "rzip.h"
 /* LZMA C Wrapper */
 #include "lzma/C/LzmaLib.h"
 
@@ -57,6 +57,53 @@
 
 static int lzo_compresses(struct stream *s);
 
+#ifndef HAS_MEMSTREAM
+# define fmemopen fake_fmemopen
+# define open_memstream fake_open_memstream
+
+static FILE *fake_fmemopen(void *buf, size_t buflen, char *mode)
+{
+        FILE *in;
+	if (strcmp(mode, "r")) 
+                fatal("fake_fmemopen only supports mode \"r\".");
+	in = tmpfile();
+	if (!in)
+	        return NULL;
+	if (1 != fwrite(buf, buflen, 1, in))
+	        return NULL;
+	rewind(in);
+        return in;
+}
+
+static FILE *fake_open_memstream(char **buf, size_t *length)
+{
+        FILE *out;
+        if (buf == NULL || length == NULL) 
+	        fatal("NULL parameter to fake_open_memstream");
+	out = tmpfile();
+	if (!out)
+	        return NULL;
+	return out;
+}
+
+static int fake_open_memstream_update_buffer(FILE *fp, uchar **buf, size_t *length)
+{
+        long original_pos = ftell(fp);
+        if (0 != fseek(fp, 0, SEEK_END)) 
+                return -1;
+  	*length = ftell(fp);
+	rewind(fp);
+	*buf = (uchar *)malloc(*length);
+	if (!*buf)
+	        return -1;
+	if (1 != fread(*buf, *length, 1, fp))
+	        return -1;
+	if (0 != fseek(fp, original_pos, SEEK_SET)) 
+	        return -1;
+	return 0;
+}
+#endif
+
 /*
   ***** COMPRESSION FUNCTIONS *****
 
@@ -84,6 +131,11 @@
 		fatal("Failed to open_memstream in zpaq_compress_buf\n");
 
 	zpipe_compress(in, out, s->buflen, (int)(control.flags & FLAG_SHOW_PROGRESS));
+
+#ifndef HAS_MEMSTREAM
+	if (0 != fake_open_memstream_update_buffer(out, &c_buf, &dlen)) 
+	        fatal("Failed to fake_open_memstream_update_buffer in zpaq_compress_buf");
+#endif
 	fclose(in);
 	fclose(out);
 
@@ -284,6 +336,10 @@
 	}
 
 	zpipe_decompress(in, out, s->buflen, (int)(control.flags & FLAG_SHOW_PROGRESS));
+#ifndef HAS_MEMSTREAM
+	if (0 != fake_open_memstream_update_buffer(out, &c_buf, &dlen)) 
+	        fatal("Failed to fake_open_memstream_update_buffer in zpaq_decompress_buf");
+#endif
 	fclose(in);
 	fclose(out);
 	free(s->buf);
